package edu.fu_tdg.lodib.test;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.Dataset;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import edu.fu_tdg.lodib.utils.Utils;
import edu.fu_tdg.lodib.validator.DataLoader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Properties;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: andreas
 * Date: 2/9/12
 * Time: 12:15 PM
 * To change this template use File | Settings | File Templates.
 */

/*
 * This class executes the Mosto SPARQL queries against an existing TDB data set on the file system
 */
public class TranslatorMostoOnTDBDataset {
    public static final String SOURCE1_DATASET_DIR = "file.source1";
    public static final String TARGET_1_FILE = "file.target1";
    public static final String SOURCE2_DATASET_DIR = "file.source2";
    public static final String TARGET_2_FILE = "file.target2";
    public static final String SOURCE3_DATASET_DIR = "file.source3";
    public static final String TARGET_3_FILE = "file.target3";
    public static final String MAPPING_1_FILE = "mapping.source1";
    public static final String MAPPING_2_FILE = "mapping.source2";
    public static final String MAPPING_3_FILE = "mapping.source3";

    public static final String MOSTO_1_CONFIG = "conf/mosto-config.properties";

    /**
     * @param args
     * @throws edu.fu_tdg.lodib.rdf_generator.RDFGeneratorException
     */
    public static void main(String[] args) throws Exception {
        Long startTime = System.currentTimeMillis();
        Properties properties = new Properties();
        properties.load(new FileInputStream(MOSTO_1_CONFIG));

        performDataTranslation(properties.getProperty(SOURCE1_DATASET_DIR), properties.getProperty(TARGET_1_FILE), properties.getProperty(MAPPING_1_FILE));
        System.out.println("Translation of source 1 took: " + (System.currentTimeMillis()-startTime)/1000.0 + "s");
        Long startTime2 = System.currentTimeMillis();
        performDataTranslation(properties.getProperty(SOURCE2_DATASET_DIR), properties.getProperty(TARGET_2_FILE), properties.getProperty(MAPPING_2_FILE));
        System.out.println("Translation of source 2 took: " + (System.currentTimeMillis()-startTime2)/1000.0 + "s");
        Long startTime3 = System.currentTimeMillis();
        performDataTranslation(properties.getProperty(SOURCE3_DATASET_DIR), properties.getProperty(TARGET_3_FILE), properties.getProperty(MAPPING_3_FILE));
        System.out.println("Translation of source 3 took: " + (System.currentTimeMillis()-startTime3)/1000.0 + "s");
        System.out.println("Translation of all sources took: " + (System.currentTimeMillis()-startTime)/1000.0 + "s");
    }

    private static void performDataTranslation(String sourceDatasetDirectory, String targetFile, String queriesFile) throws Exception {
        Dataset source = DataLoader.openDataset(sourceDatasetDirectory);
        OntModel target = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM, DataLoader.createTmpTDBModel());

        Set<Query> queries = Utils.parseJenaSPARQLQueries(queriesFile);

        performDataTranslation(source, target, queries);

        target.write(new FileOutputStream(new File(targetFile)), Utils.N_TRIPLE);
    }

    private static void performDataTranslation(Dataset source, OntModel target, Collection<Query> mappings) {
        for (Query m : mappings)
            QueryExecutionFactory.create(m, source).execConstruct(target);
    }

}
