package edu.fu_tdg.lodib.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.Dataset;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.Syntax;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

import edu.fu_tdg.lodib.mapping.DataTranslationException;
import edu.fu_tdg.lodib.validator.DataLoader;

public class Utils {
	public static final String RDF_TYPE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
	public static final String GT = "<";
	public static final String LT = ">";
	public static final String SPACE = " ";
	public static final String DOT = ".";
	public static final String QUOTATION = "\"";
	public static final String HAT = "^";
	public static final String AT = "@";
	public static final String NEWLINE = "\n";
	public static final String DATE_PATTERN = "yyyy-MM-dd";
	public static final DateFormat DATE_FORMAT = new SimpleDateFormat(Utils.DATE_PATTERN);
	public static final String WHERE = "WHERE";
	public static final String CONSTRUCT = "CONSTRUCT";
	public static final String SELECT = "SELECT";
	public static final String OPEN_PARENTHESIS = "(";
	public static final String CLOSE_PARENTHESIS = ")";
	public static final String OPEN_CURLY_BRACKET = "{";
	public static final String CLOSE_CURLY_BRACKET = "}";
	public static final String N_TRIPLE = "N-TRIPLE";
	public static final String OPTIONAL = "OPTIONAL";
	public static final String BOUND = "BOUND";
	public static final String FILTER = "FILTER";
	public static final String LET = "LET";
	public static final String ASSIGN = ":=";
	public static final String COMMA = ",";
	public static final String PREFIX = "PREFIX";
	public static final String COUNT = "COUNT";
	public static final String GROUP_BY = "GROUP BY";
	public static final String AS = "AS";
	public static final String SHARP = "#";
	
	@SuppressWarnings("unchecked")
	public static <T> T unmarshall(Class<T> docClass, InputStream inputStream) throws JAXBException {
	    String packageName = docClass.getPackage().getName();
		JAXBContext jc = JAXBContext.newInstance( packageName );
	    Unmarshaller u = jc.createUnmarshaller();
	    JAXBElement<T> doc = (JAXBElement<T>) u.unmarshal( inputStream );
	    return doc.getValue();
	}
	
	public static String decode(String str, Map<String, String> namespaces) {
		if (namespaces != null)
			for (String prf : namespaces.keySet())
				str = str.replace(prf, namespaces.get(prf));
		return str;
	}
	
	public static String toURI(String str, Map<String, String> namespaces) {
		return Utils.GT + decode(str, namespaces) + Utils.LT;
	}
	
	public static void addTriple(Writer w, String subject, String predicate, String object) throws IOException {
		w.append(subject + Utils.SPACE + predicate + Utils.SPACE + object + Utils.DOT);
		w.append(Utils.NEWLINE);
	}
	
	public static String createLiteralType(String value, String type, Map<String, String> namespaces) {
		StringBuffer buf = new StringBuffer();
		buf.append(createLiteralUntyped(value));
		if (type != null) {
			buf.append(HAT);
			buf.append(HAT);
			buf.append(toURI(type, namespaces));
		}
		return buf.toString();
	}

    public static String createLiteralUntyped(String value) {
        StringBuffer buf = new StringBuffer();
		buf.append(QUOTATION);
		buf.append(value);
		buf.append(QUOTATION);
        return buf.toString();
    }
	
	public static String createLiteralLang(String value, String lang) {
		StringBuffer buf = new StringBuffer();
        buf.append(createLiteralUntyped(value));
		buf.append(AT);
		buf.append(lang);
		return buf.toString();
	}
	
	public static String buildRDFTypeTriplePattern(String var, String className) {
		return var + SPACE + toURI(RDF_TYPE, null) + SPACE + className + DOT;
	}
	
	public static String buildPropertyTriplePattern(String var1, String property, String var2) {
		return var1 + SPACE + property + SPACE + var2 + DOT;
	}
	
	public static String buildOptionalTriple(String triple) {
		return OPTIONAL + OPEN_CURLY_BRACKET + triple + CLOSE_CURLY_BRACKET + DOT;
	}
	
	public static String buildBoundVariable(String var) {
		return BOUND + OPEN_PARENTHESIS + var + CLOSE_PARENTHESIS;
	}
	
	public static String buildFilter(String exp) {
		return FILTER + OPEN_PARENTHESIS + exp + CLOSE_PARENTHESIS;
	}
	
	public static String buildSPARQLConstruct(String where, String construct) {
		return CONSTRUCT + OPEN_CURLY_BRACKET + construct + CLOSE_CURLY_BRACKET + WHERE + OPEN_CURLY_BRACKET + where + CLOSE_CURLY_BRACKET;
	}
	
	public static String buildSPARQLSelect(String where, List<String> variables) {
		StringBuffer buf = new StringBuffer(SELECT);
		for (String v : variables) {
			buf.append(SPACE);
			buf.append(v);
		}
		buf.append(SPACE);
		buf.append(OPEN_CURLY_BRACKET);
		buf.append(where);
		buf.append(CLOSE_CURLY_BRACKET);
		return buf.toString();
	}
	
	public static String buildAssignment(String left, String right) {
		return LET + OPEN_PARENTHESIS + left + SPACE + ASSIGN + SPACE + right + CLOSE_PARENTHESIS;
	}
	
	public static String buildFunction(String var, String function) {
		List<String> params = new ArrayList<String>();
		params.add(var);
		return buildFunction(params, function);
	}
	
	public static String buildFunction(List<String> parameters, String function) {
		StringBuffer buf = new StringBuffer(toURI(function, null) + OPEN_PARENTHESIS);
		
		for (String p : parameters) {
			buf.append(p);
			if (parameters.indexOf(p) != parameters.size() - 1)
				buf.append(COMMA);
		}
		buf.append(CLOSE_PARENTHESIS);
		return buf.toString();
	}
	
	public static String buildCount(String var) {
		return Utils.COUNT + OPEN_PARENTHESIS + var + CLOSE_PARENTHESIS;
	}
	
	public static String buildAs(String var, String expression) {
		return OPEN_PARENTHESIS + expression + SPACE + AS + SPACE + var + CLOSE_PARENTHESIS;
	}
	
	public static String buildGroupBy(List<String> variables) {
		StringBuffer buf = new StringBuffer(GROUP_BY);
		for (String v : variables) {
			buf.append(SPACE);
			buf.append(v);
		}
		return buf.toString();
	}
	
	public static void copy(File src, File dst, boolean append) throws IOException {
		InputStream in = new FileInputStream(src);
		OutputStream out = new FileOutputStream(dst, append);

		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) out.write(buf, 0, len);
		
		in.close();
		out.close();
	}
	
	public static void executeJenaQuery(File in, File out, String query) throws DataTranslationException {
		try {
			OntModel source = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM);
			source.read(new FileInputStream(in), "", Utils.N_TRIPLE);
			
			OntModel target = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM);
			QueryExecutionFactory.create(query, Syntax.syntaxARQ, source).execConstruct(target);
			
			target.write(new FileOutputStream(out), Utils.N_TRIPLE);
		} catch (IOException oops) {
			throw new DataTranslationException("Error when executing this mapping: " + oops);
		}
	}

    public static void executeJenaQuery(Dataset source, File out, String queryString) throws DataTranslationException {
		try {
            Model model = DataLoader.createTmpTDBModel();
            QueryExecutionFactory.create(queryString, Syntax.syntaxARQ, source).execConstruct(model);
            model.write(new FileOutputStream(out), Utils.N_TRIPLE);
            model.close();
		} catch (IOException oops) {
			throw new DataTranslationException("Error when executing this mapping: " + oops);
		}
	}
	
	public static Set<Query> parseJenaSPARQLQueries(String file, Syntax syntax) throws IOException {
		Set<Query> sparqlQueries = new HashSet<Query>();
		
		BufferedReader input = new BufferedReader(new FileReader(new File(file)));
		
		String line = null;
		StringBuffer sb = new StringBuffer();
        
        while (( line = input.readLine()) != null) {
        	// End of query
        	if (line.length() == 0) {
        		if (sb.toString().length() > 0) {
        			sparqlQueries.add(parseJenaSPARQLQuery(sb.toString(), syntax));
        			sb = new StringBuffer();
        		}
        	}
        	else if (!line.startsWith(SHARP))
        		sb.append(line);
        }
        
        // Add the last one
        if (sb.length() > 0)
        	sparqlQueries.add(parseJenaSPARQLQuery(sb.toString(), syntax));
		
		return sparqlQueries;
	}
	
	public static Query parseJenaSPARQLQuery(String q, Syntax syntax) {
		return QueryFactory.create(q, syntax);
	}
}
